@@ -20,16 +20,6 @@
 from merlin.models.loader.tf_utils import configure_tensorflow
 from merlin.models.tf.core.index import IndexBlock, TopKIndexBlock
 from merlin.models.tf.core.tabular import AsTabular, Filter, TabularBlock
-from merlin.models.tf.core.transformations import (
-    AsDenseFeatures,
-    AsRaggedFeatures,
-    AsSparseFeatures,
-    CategoryEncoding,
-    ExpandDims,
-    HashedCross,
-    HashedCrossAll,
-    LabelToOneHot,
-)
 
 configure_tensorflow()
 
@@ -43,7 +33,11 @@ from merlin.models.loader.tf_utils import configure_tensorflow
 from merlin.models.tf.blocks.cross import CrossBlock
 from merlin.models.tf.blocks.dlrm import DLRMBlock
 from merlin.models.tf.blocks.experts import CGCBlock, MMOEBlock, MMOEGate
-from merlin.models.tf.blocks.interaction import DotProductInteraction, FMPairwiseInteraction
+from merlin.models.tf.blocks.interaction import (
+    DotProductInteraction,
+    FMBlock,
+    FMPairwiseInteraction,
+)
 from merlin.models.tf.blocks.mlp import DenseResidualBlock, MLPBlock
 from merlin.models.tf.blocks.optimizer import (
     LazyAdam,
@@ -81,10 +75,9 @@ from merlin.models.tf.core.combinators import (
     ResidualBlock,
     SequentialBlock,
 )
-from merlin.models.tf.data_augmentation.noise import StochasticSwapNoise
-from merlin.models.tf.dataset import sample_batch
+from merlin.models.tf.core.encoder import EmbeddingEncoder, Encoder, TopKEncoder
 from merlin.models.tf.inputs.base import InputBlock, InputBlockV2
-from merlin.models.tf.inputs.continuous import ContinuousFeatures
+from merlin.models.tf.inputs.continuous import Continuous, ContinuousFeatures, ContinuousProjection
 from merlin.models.tf.inputs.embedding import (
     AverageEmbeddingsByWeightFeature,
     ContinuousEmbedding,
@@ -96,6 +89,7 @@ from merlin.models.tf.inputs.embedding import (
     SequenceEmbeddingFeatures,
     TableConfig,
 )
+from merlin.models.tf.loader import KerasSequenceValidator, Loader, sample_batch
 from merlin.models.tf.losses import LossType
 from merlin.models.tf.metrics.topk import (
     AvgPrecisionAt,
@@ -106,28 +100,68 @@ from merlin.models.tf.metrics.topk import (
     TopKMetricsAggregator,
 )
 from merlin.models.tf.models import benchmark
-from merlin.models.tf.models.base import BaseModel, Model, RetrievalModel
+from merlin.models.tf.models.base import BaseModel, Model, RetrievalModel, RetrievalModelV2
 from merlin.models.tf.models.ranking import DCNModel, DeepFMModel, DLRMModel, WideAndDeepModel
 from merlin.models.tf.models.retrieval import (
     MatrixFactorizationModel,
     TwoTowerModel,
     YoutubeDNNRetrievalModel,
 )
+from merlin.models.tf.outputs.base import ModelOutput
+from merlin.models.tf.outputs.classification import BinaryOutput, CategoricalOutput
+from merlin.models.tf.outputs.contrastive import ContrastiveOutput
+from merlin.models.tf.outputs.regression import RegressionOutput
+from merlin.models.tf.outputs.sampling.base import Candidate, CandidateSampler
+from merlin.models.tf.outputs.sampling.in_batch import InBatchSamplerV2
+from merlin.models.tf.outputs.sampling.popularity import PopularityBasedSamplerV2
+from merlin.models.tf.outputs.topk import TopKOutput
 from merlin.models.tf.prediction_tasks.base import ParallelPredictionBlock, PredictionTask
 from merlin.models.tf.prediction_tasks.classification import (
     BinaryClassificationTask,
     MultiClassClassificationTask,
 )
 from merlin.models.tf.prediction_tasks.multi import PredictionTasks
-from merlin.models.tf.prediction_tasks.next_item import NextItemPredictionTask
 from merlin.models.tf.prediction_tasks.regression import RegressionTask
 from merlin.models.tf.prediction_tasks.retrieval import ItemRetrievalTask
-from merlin.models.tf.predictions.base import PredictionBlock
-from merlin.models.tf.predictions.classification import BinaryPrediction, CategoricalPrediction
-from merlin.models.tf.predictions.regression import RegressionPrediction
-from merlin.models.tf.predictions.sampling.base import Items, ItemSamplerV2
-from merlin.models.tf.predictions.sampling.in_batch import InBatchSamplerV2
-from merlin.models.tf.predictions.sampling.popularity import PopularityBasedSamplerV2
+from merlin.models.utils.dependencies import is_transformers_available
+
+if is_transformers_available():
+    from merlin.models.tf.transformers.block import (
+        AlbertBlock,
+        BertBlock,
+        GPT2Block,
+        RobertaBlock,
+        TransformerBlock,
+        XLNetBlock,
+    )
+    from merlin.models.tf.transformers.transforms import (
+        AttentionWeights,
+        HiddenStates,
+        LastHiddenState,
+        LastHiddenStateAndAttention,
+    )
+
+from merlin.models.tf.transforms.features import (
+    CategoryEncoding,
+    HashedCross,
+    HashedCrossAll,
+    ToDense,
+    ToOneHot,
+    ToSparse,
+    ToTarget,
+)
+from merlin.models.tf.transforms.noise import StochasticSwapNoise
+from merlin.models.tf.transforms.regularization import L2Norm
+from merlin.models.tf.transforms.sequence import (
+    ReplaceMaskedEmbeddings,
+    SequenceMaskLast,
+    SequenceMaskRandom,
+    SequencePredictLast,
+    SequencePredictNext,
+    SequencePredictRandom,
+    SequenceTargetAsInput,
+)
+from merlin.models.tf.transforms.tensor import ExpandDims, ListToDense, ListToRagged, ListToSparse
 from merlin.models.tf.utils import repr_utils
 from merlin.models.tf.utils.tf_utils import TensorInitializer
 
@@ -149,6 +183,9 @@ __all__ = [
     "SequentialBlock",
     "ResidualBlock",
     "DualEncoderBlock",
+    "TopKEncoder",
+    "Encoder",
+    "EmbeddingEncoder",
     "CrossBlock",
     "DLRMBlock",
     "MLPBlock",
@@ -161,6 +198,8 @@ __all__ = [
     "DenseResidualBlock",
     "TabularBlock",
     "ContinuousFeatures",
+    "Continuous",
+    "ContinuousProjection",
     "EmbeddingFeatures",
     "SequenceEmbeddingFeatures",
     "EmbeddingOptions",
@@ -173,9 +212,12 @@ __all__ = [
     "TwoTowerBlock",
     "MatrixFactorizationBlock",
     "QueryItemIdsEmbeddingsBlock",
-    "AsDenseFeatures",
-    "AsRaggedFeatures",
-    "AsSparseFeatures",
+    "ListToDense",
+    "ListToRagged",
+    "ListToSparse",
+    "ToSparse",
+    "ToDense",
+    "ToTarget",
     "CategoryEncoding",
     "HashedCross",
     "HashedCrossAll",
@@ -188,11 +230,13 @@ __all__ = [
     "StackFeatures",
     "DotProductInteraction",
     "FMPairwiseInteraction",
-    "LabelToOneHot",
-    "PredictionBlock",
-    "BinaryPrediction",
-    "RegressionPrediction",
-    "CategoricalPrediction",
+    "FMBlock",
+    "ToOneHot",
+    "ModelOutput",
+    "BinaryOutput",
+    "RegressionOutput",
+    "CategoricalOutput",
+    "ContrastiveOutput",
     "PredictionTask",
     "BinaryClassificationTask",
     "MultiClassClassificationTask",
@@ -204,7 +248,6 @@ __all__ = [
     "OptimizerBlocks",
     "ItemRetrievalTask",
     "ItemRetrievalScorer",
-    "NextItemPredictionTask",
     "NDCGAt",
     "PrecisionAt",
     "MRRAt",
@@ -214,11 +257,13 @@ __all__ = [
     "TopKMetricsAggregator",
     "Model",
     "RetrievalModel",
+    "RetrievalModelV2",
     "InputBlock",
     "InputBlockV2",
     "PredictionTasks",
     "StochasticSwapNoise",
     "ExpandDims",
+    "L2Norm",
     "NoOp",
     "ItemSampler",
     "EmbeddingWithMetadata",
@@ -236,4 +281,14 @@ __all__ = [
     "LossType",
     "sample_batch",
     "TensorInitializer",
+    "BroadcastToSequence",
+    "Loader",
+    "KerasSequenceValidator",
+    "SequencePredictNext",
+    "SequencePredictLast",
+    "SequencePredictRandom",
+    "SequenceTargetAsInput",
+    "SequenceMaskLast",
+    "SequenceMaskRandom",
+    "ReplaceMaskedEmbeddings",
 ]
