@@ -1,4 +1,5 @@
 import abc
+import collections
 import copy
 from typing import Dict, List, Optional, Sequence, Union, overload
 
@@ -140,6 +141,12 @@ class TabularBlock(Block):
         if schema:
             self.set_schema(schema)
 
+    def select_by_tag(self, tags: Tags) -> Optional["TabularBlock"]:
+        selected_schema = self.schema.select_by_tag(tags)
+        if selected_schema:
+            return TabularBlock(schema=selected_schema)
+        return None
+
     @property
     def is_input(self) -> bool:
         return self._is_input
@@ -541,7 +548,11 @@ class Filter(TabularBlock):
         self.pop = pop
         self.add_to_context = add_to_context
 
+        if isinstance(inputs, Schema):
+            self.set_schema(schema=inputs)
+
     def set_schema(self, schema=None):
+
         out = super().set_schema(schema)
 
         if isinstance(self.feature_names, Tags):
@@ -597,6 +608,19 @@ class Filter(TabularBlock):
 
         return config
 
+    def select_by_tag(self, tags: Tags) -> Optional["Filter"]:
+        if isinstance(self.feature_names, Tags):
+            schema = self.schema.select_by_tag(self.feature_names).select_by_tag(tags)
+        elif isinstance(self.feature_names, collections.Sequence):
+            schema = self.schema.select_by_name(self.feature_names).select_by_tag(tags)
+        else:
+            raise RuntimeError(
+                f"Unexpected type {type(self.feature_names)} encountered in 'Filter.feature_names'."
+            )
+        if not schema:
+            return
+        return Filter(schema)
+
 
 @tf.keras.utils.register_keras_serializable(package="merlin.models")
 class AsTabular(tf.keras.layers.Layer):
