@@ -13,10 +13,11 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
+import collections
 import inspect
 from copy import copy, deepcopy
 from dataclasses import dataclass
-from typing import Any, Callable, Dict, Optional, Type, Union
+from typing import Any, Callable, Dict, Optional, Sequence, Type, Union
 
 import tensorflow as tf
 from tensorflow.keras import backend
@@ -35,13 +36,18 @@ from merlin.models.tf.core.tabular import (
     TabularAggregationType,
     TabularBlock,
 )
-from merlin.models.tf.core.transformations import AsDenseFeatures, AsSparseFeatures
+from merlin.models.tf.transforms.tensor import ListToDense, ListToSparse
 
 # pylint has issues with TF array ops, so disable checks until fixed:
 # https://github.com/PyCQA/pylint/issues/3613
 # pylint: disable=no-value-for-parameter, unexpected-keyword-arg
 from merlin.models.tf.typing import TabularData
-from merlin.models.tf.utils.tf_utils import call_layer, df_to_tensor, list_col_to_ragged
+from merlin.models.tf.utils.tf_utils import (
+    call_layer,
+    df_to_tensor,
+    list_col_to_ragged,
+    tensor_to_df,
+)
 from merlin.models.utils import schema_utils
 from merlin.models.utils.doc_utils import docstring_parameter
 from merlin.models.utils.schema_utils import (
@@ -244,6 +250,34 @@ class EmbeddingTable(EmbeddingTableBase):
         self.sequence_combiner = sequence_combiner
         self.supports_masking = True
 
+    def select_by_tag(self, tags: Union[Tags, Sequence[Tags]]) -> Optional["EmbeddingTable"]:
+        """Select features in EmbeddingTable by tags.
+
+        Since an EmbeddingTable can be a shared-embedding table, this method filters
+        the schema for features that match the tags.
+
+        If none of the features match the tags, it will return None.
+
+        Parameters
+        ----------
+        tags: Union[Tags, Sequence[Tags]]
+            A list of tags.
+
+        Returns
+        -------
+        An EmbeddingTable if the tags match. If no features match, it returns None.
+        """
+        if not isinstance(tags, collections.Sequence):
+            tags = [tags]
+
+        selected_schema = self.schema.select_by_tag(tags)
+        if not selected_schema:
+            return
+        config = self.get_config()
+        config["schema"] = schema_utils.schema_to_tensorflow_metadata_json(selected_schema)
+        embedding_table = EmbeddingTable.from_config(config, table=self.table)
+        return embedding_table
+
     @classmethod
     def from_pretrained(
         cls,
@@ -252,9 +286,8 @@ class EmbeddingTable(EmbeddingTableBase):
         name=None,
         col_schema=None,
         **kwargs,
-    ):
+    ) -> "EmbeddingTable":
         """Create From pre-trained embeddings from a Dataset or DataFrame.
-
         Parameters
         ----------
         data : Union[Dataset, DataFrameType]
@@ -284,6 +317,35 @@ class EmbeddingTable(EmbeddingTableBase):
             **kwargs,
         )
 
+    @classmethod
+    def from_dataset(
+        cls,
+        data: Union[Dataset, DataFrameType],
+        trainable=True,
+        name=None,
+        col_schema=None,
+        **kwargs,
+    ) -> "EmbeddingTable":
+        """Create From pre-trained embeddings from a Dataset or DataFrame.
+        Parameters
+        ----------
+        data : Union[Dataset, DataFrameType]
+            A dataset containing the pre-trained embedding weights
+        trainable : bool
+            Whether the layer should be trained or not.
+        name : str
+            The name of the layer.
+        """
+        return cls.from_pretrained(
+            data, trainable=trainable, name=name, col_schema=col_schema, **kwargs
+        )
+
+    def to_dataset(self, gpu=None) -> merlin.io.Dataset:
+        return merlin.io.Dataset(self.to_df(gpu=gpu))
+
+    def to_df(self, gpu=None):
+        return tensor_to_df(self.table.embeddings, gpu=gpu)
+
     def _maybe_build(self, inputs):
         """Creates state between layer instantiation and layer call.
         Invoked automatically before the first execution of `call()`.
@@ -387,8 +449,11 @@ class EmbeddingTable(EmbeddingTableBase):
         return self.compute_output_shape(input_shapes)
 
     @classmethod
-    def from_config(cls, config):
-        config["table"] = tf.keras.layers.deserialize(config["table"])
+    def from_config(cls, config, table=None):
+        if table:
+            config["table"] = table
+        else:
+            config["table"] = tf.keras.layers.deserialize(config["table"])
         if "combiner-layer" in config:
             config["sequence_combiner"] = tf.keras.layers.deserialize(config.pop("combiner-layer"))
 
@@ -401,7 +466,6 @@ class EmbeddingTable(EmbeddingTableBase):
             config["combiner-layer"] = tf.keras.layers.serialize(self.sequence_combiner)
         else:
             config["sequence_combiner"] = self.sequence_combiner
-
         return config
 
 
@@ -647,7 +711,7 @@ class EmbeddingFeatures(TabularBlock):
         **kwargs,
     ):
         if add_default_pre:
-            embedding_pre = [Filter(list(feature_config.keys())), AsSparseFeatures()]
+            embedding_pre = [Filter(list(feature_config.keys())), ListToSparse()]
             pre = [embedding_pre, pre] if pre else embedding_pre  # type: ignore
         self.feature_config = feature_config
         self.l2_reg = l2_reg
@@ -1006,7 +1070,7 @@ class SequenceEmbeddingFeatures(EmbeddingFeatures):
         **kwargs,
     ):
         if add_default_pre:
-            embedding_pre = [Filter(list(feature_config.keys())), AsDenseFeatures(max_seq_length)]
+            embedding_pre = [Filter(list(feature_config.keys())), ListToDense(max_seq_length)]
             pre = [embedding_pre, pre] if pre else embedding_pre  # type: ignore
 
         super().__init__(
