@@ -9,23 +9,25 @@ from typing import TYPE_CHECKING, Dict, List, Optional, Protocol, Sequence, Unio
 
 import six
 import tensorflow as tf
+from keras.engine.compile_utils import MetricsContainer
 from keras.utils.losses_utils import cast_losses_to_common_dtype
 from packaging import version
 from tensorflow.keras.utils import unpack_x_y_sample_weight
 
 import merlin.io
 from merlin.models.tf.core.base import Block, ModelContext, PredictionOutput, is_input_block
-from merlin.models.tf.core.combinators import SequentialBlock
-from merlin.models.tf.core.prediction import Prediction, PredictionContext
+from merlin.models.tf.core.combinators import ParallelBlock, SequentialBlock
+from merlin.models.tf.core.prediction import Prediction, PredictionContext, TensorLike
 from merlin.models.tf.core.tabular import TabularBlock
-from merlin.models.tf.core.transformations import AsRaggedFeatures
-from merlin.models.tf.dataset import BatchedDataset
 from merlin.models.tf.inputs.base import InputBlock
+from merlin.models.tf.loader import Loader
 from merlin.models.tf.losses.base import loss_registry
 from merlin.models.tf.metrics.topk import TopKMetricsAggregator, filter_topk_metrics, split_metrics
 from merlin.models.tf.models.utils import parse_prediction_tasks
+from merlin.models.tf.outputs.base import ModelOutput
+from merlin.models.tf.outputs.contrastive import ContrastiveOutput
 from merlin.models.tf.prediction_tasks.base import ParallelPredictionBlock, PredictionTask
-from merlin.models.tf.predictions.base import ContrastivePredictionBlock, PredictionBlock
+from merlin.models.tf.transforms.tensor import ListToRagged
 from merlin.models.tf.typing import TabularData
 from merlin.models.tf.utils.search_utils import find_all_instances_in_layers
 from merlin.models.tf.utils.tf_utils import (
@@ -34,9 +36,10 @@ from merlin.models.tf.utils.tf_utils import (
     maybe_serialize_keras_objects,
 )
 from merlin.models.utils.dataset import unique_rows_by_features
-from merlin.schema import Schema, Tags
+from merlin.schema import ColumnSchema, Schema, Tags
 
 if TYPE_CHECKING:
+    from merlin.models.tf.core.encoder import Encoder
     from merlin.models.tf.core.index import TopKIndexBlock
 
 
@@ -300,7 +303,7 @@ class BaseModel(tf.keras.Model):
         )
 
         num_v1_blocks = len(self.prediction_tasks)
-        num_v2_blocks = len(self.prediction_blocks)
+        num_v2_blocks = len(self.model_outputs)
 
         if num_v1_blocks > 1 and num_v2_blocks > 1:
             raise ValueError(
@@ -311,15 +314,7 @@ class BaseModel(tf.keras.Model):
         if num_v1_blocks > 0:
             self.output_names = [task.task_name for task in self.prediction_tasks]
         else:
-            self.output_names = [block.full_name for block in self.prediction_blocks]
-            negative_sampling = kwargs.pop("negative_sampling", None)
-            if negative_sampling:
-                if not isinstance(self.prediction_blocks[0], ContrastivePredictionBlock):
-                    raise ValueError(
-                        "Negative sampling strategy can be used only with a"
-                        " `ContrastivePredictionBlock` prediction block"
-                    )
-                self.prediction_blocks[0].compile(negative_sampling=negative_sampling)
+            self.output_names = [block.full_name for block in self.model_outputs]
 
         # This flag will make Keras change the metric-names which is not needed in v2
         from_serialized = kwargs.pop("from_serialized", num_v2_blocks > 0)
@@ -347,9 +342,12 @@ class BaseModel(tf.keras.Model):
         elif isinstance(metrics, (list, tuple)):
             # Retrieve top-k metrics & wrap them in TopKMetricsAggregator
             topk_metrics, topk_aggregators, other_metrics = split_metrics(metrics)
-            if len(topk_metrics) > 0:
-                topk_aggregators.append(TopKMetricsAggregator(*topk_metrics))
             metrics = other_metrics + topk_aggregators
+            if len(topk_metrics) > 0:
+                if len(topk_metrics) == 1:
+                    metrics.append(topk_metrics[0])
+                else:
+                    metrics.append(TopKMetricsAggregator(*topk_metrics))
 
             if num_v1_blocks > 0:
                 if num_v1_blocks == 1:
@@ -358,21 +356,23 @@ class BaseModel(tf.keras.Model):
                     for i, task in enumerate(self.prediction_tasks):
                         out[task.task_name] = metrics[i]
             else:
-                if len(self.prediction_blocks) == 1:
-                    out[self.prediction_blocks[0].full_name] = metrics
+                if len(self.model_outputs) == 1:
+                    out[self.model_outputs[0].full_name] = metrics
                 else:
-                    for i, block in enumerate(self.prediction_blocks):
+                    for i, block in enumerate(self.model_outputs):
                         out[block.full_name] = metrics[i]
 
         elif metrics is None:
             for task_name, task in self.prediction_tasks_by_name().items():
                 out[task_name] = [m() if inspect.isclass(m) else m for m in task.DEFAULT_METRICS]
 
-            for prediction_name, prediction_block in self.predictions_by_name().items():
+            for prediction_name, prediction_block in self.outputs_by_name().items():
                 out[prediction_name] = prediction_block.default_metrics_fn()
-                if len(self.prediction_blocks) > 1:
+                if len(self.model_outputs) > 1:
                     for metric in out[prediction_name]:
                         metric._name = "/".join([prediction_block.full_name, metric.name])
+        else:
+            out = metrics
 
         return out
 
@@ -392,10 +392,10 @@ class BaseModel(tf.keras.Model):
                     for i, task in enumerate(self.prediction_tasks):
                         out[task.task_name] = weighted_metrics[i]
             else:
-                if len(self.prediction_blocks) == 1:
-                    out[self.prediction_blocks[0].full_name] = weighted_metrics
+                if len(self.model_outputs) == 1:
+                    out[self.model_outputs[0].full_name] = weighted_metrics
                 else:
-                    for i, block in enumerate(self.prediction_blocks):
+                    for i, block in enumerate(self.model_outputs):
                         out[block.full_name] = weighted_metrics[i]
 
         return out
@@ -406,15 +406,15 @@ class BaseModel(tf.keras.Model):
         if isinstance(loss, (tf.keras.losses.Loss, str)):
             if len(self.prediction_tasks) == 1:
                 out = {task.task_name: loss for task in self.prediction_tasks}
-            elif len(self.prediction_blocks) == 1:
-                out = {task.name: loss for task in self.prediction_blocks}
+            elif len(self.model_outputs) == 1:
+                out = {task.name: loss for task in self.model_outputs}
 
         # If loss is not provided, use the defaults from the prediction-tasks.
         if not loss:
             for task_name, task in self.prediction_tasks_by_name().items():
                 out[task_name] = task.DEFAULT_LOSS
 
-            for task_name, task in self.predictions_by_name().items():
+            for task_name, task in self.outputs_by_name().items():
                 out[task_name] = task.default_loss
 
         for key in out:
@@ -454,15 +454,15 @@ class BaseModel(tf.keras.Model):
         return outputs
 
     @property
-    def prediction_blocks(self) -> List[PredictionBlock]:
-        results = find_all_instances_in_layers(self, PredictionBlock)
+    def model_outputs(self) -> List[ModelOutput]:
+        results = find_all_instances_in_layers(self, ModelOutput)
 
         return results
 
-    def predictions_by_name(self) -> Dict[str, PredictionBlock]:
-        return {task.full_name: task for task in self.prediction_blocks}
+    def outputs_by_name(self) -> Dict[str, ModelOutput]:
+        return {task.full_name: task for task in self.model_outputs}
 
-    def predictions_by_target(self) -> Dict[str, List[PredictionBlock]]:
+    def outputs_by_target(self) -> Dict[str, List[ModelOutput]]:
         """Method to index the model's prediction blocks by target names.
 
         Returns
@@ -470,8 +470,8 @@ class BaseModel(tf.keras.Model):
         Dict[str, List[PredictionBlock]]
             List of prediction blocks.
         """
-        outputs: Dict[str, List[PredictionBlock]] = {}
-        for task in self.prediction_blocks:
+        outputs: Dict[str, List[ModelOutput]] = {}
+        for task in self.model_outputs:
             if task.target in outputs:
                 if isinstance(outputs[task.target], list):
                     outputs[task.target].append(task)
@@ -491,7 +491,8 @@ class BaseModel(tf.keras.Model):
         **kwargs,
     ) -> Union[Prediction, PredictionOutput]:
         """Apply the model's call method during Train or Test modes and prepare
-        Prediction (v2) or PredictionOutput (v1 - depreciated) objects
+        Prediction (v2) or PredictionOutput (v1 -
+        depreciated) objects
 
         Parameters
         ----------
@@ -518,7 +519,7 @@ class BaseModel(tf.keras.Model):
             testing=testing,
             **kwargs,
         )
-        if not (self.prediction_tasks or self.prediction_blocks):
+        if not (self.prediction_tasks or self.model_outputs):
             return PredictionOutput(forward, y)
 
         predictions, targets, sample_weights, output = {}, {}, {}, None
@@ -544,6 +545,8 @@ class BaseModel(tf.keras.Model):
                 predictions[task.task_name] = task_x
                 sample_weights[task.task_name] = task_sample_weight
 
+            self.adjust_predictions_and_targets(predictions, targets)
+
             if len(predictions) == 1 and len(targets) == 1:
                 predictions = list(predictions.values())[0]
                 targets = list(targets.values())[0]
@@ -555,7 +558,7 @@ class BaseModel(tf.keras.Model):
                 return PredictionOutput(predictions, targets, sample_weight=sample_weights)
 
         # V2
-        for task in self.prediction_blocks:
+        for task in self.model_outputs:
             task_x = forward
             if isinstance(forward, dict) and task.full_name in forward:
                 task_x = forward[task.full_name]
@@ -574,13 +577,78 @@ class BaseModel(tf.keras.Model):
             predictions[task.full_name] = task_x
             sample_weights[task.full_name] = task_sample_weight
 
+            self.adjust_predictions_and_targets(predictions, targets)
+
         return Prediction(predictions, targets, sample_weights)
 
+    def adjust_predictions_and_targets(
+        self,
+        predictions: Dict[str, TensorLike],
+        targets: Optional[Union[tf.Tensor, Dict[str, tf.Tensor]]],
+    ):
+        """Adjusts the predctions and targets, doing the following transformations
+        if the target is provided:
+        - Converts ragged targets (and their masks) to dense, so that they are compatible
+        with most losses and metrics
+        - Copies the targets mask to predictions mask, if defined
+        - One-hot encode targets if their tf.rank(targest) == tf.rank(predictions)-1
+        - Ensures targets has the same shape and dtype as predicitnos
+
+        Parameters
+        ----------
+        predictions : Dict[str, TensorLike]
+            A dict with predictions for the tasks
+        targets : Optional[Union[tf.Tensor, Dict[str, tf.Tensor]]]
+            A dict with targets for the tasks
+        """
+        if targets is None:
+            return
+
+        for k in targets:
+            # Convert ragged targets (and ragged mask) to dense
+            if isinstance(targets[k], tf.RaggedTensor):
+                dense_target_mask = None
+                if getattr(targets[k], "_keras_mask", None) is not None:
+                    dense_target_mask = targets[k]._keras_mask.to_tensor()
+                targets[k] = targets[k].to_tensor()
+                if dense_target_mask is not None:
+                    targets[k]._keras_mask = dense_target_mask
+
+            if getattr(targets[k], "_keras_mask", None) is not None:
+                # Copies the mask from the targets to the predictions
+                # because Keras considers the prediction mask in loss
+                # and metrics computation
+                predictions[k]._keras_mask = targets[k]._keras_mask
+
+            # Ensuring targets and preds have the same dtype
+            targets[k] = tf.cast(targets[k], predictions[k].dtype)
+
+            # Ensuring targets are one-hot encoded if they are not
+            targets[k] = tf.cond(
+                tf.rank(targets[k]) == tf.rank(predictions[k]) - 1,
+                lambda: tf.one_hot(
+                    tf.cast(targets[k], tf.int32),
+                    tf.shape(predictions[k])[-1],
+                    dtype=predictions[k].dtype,
+                ),
+                lambda: targets[k],
+            )
+            # Makes target shape equal to the predictions tensor, as shape is lost after tf.cond
+            targets[k] = tf.reshape(targets[k], tf.shape(predictions[k]))
+
     def train_step(self, data):
         """Custom train step using the `compute_loss` method."""
 
         with tf.GradientTape() as tape:
             x, y, sample_weight = unpack_x_y_sample_weight(data)
+
+            if getattr(self, "train_pre", None):
+                out = call_layer(self.train_pre, x, targets=y, features=x, training=True)
+                if isinstance(out, Prediction):
+                    x, y = out.outputs, out.targets
+                else:
+                    x = out
+
             outputs = self.call_train_test(x, y, sample_weight=sample_weight, training=True)
             loss = self.compute_loss(x, outputs.targets, outputs.predictions, outputs.sample_weight)
 
@@ -589,7 +657,8 @@ class BaseModel(tf.keras.Model):
         # Run backwards pass.
         self.optimizer.minimize(loss, self.trainable_variables, tape=tape)
 
-        metrics = self.compute_metrics(outputs, training=True)
+        metrics = self.train_compute_metrics(outputs, self.compiled_metrics)
+
         # Adding regularization loss to metrics
         metrics["regularization_loss"] = tf.reduce_sum(cast_losses_to_common_dtype(self.losses))
 
@@ -599,6 +668,14 @@ class BaseModel(tf.keras.Model):
         """Custom test step using the `compute_loss` method."""
 
         x, y, sample_weight = unpack_x_y_sample_weight(data)
+
+        if getattr(self, "test_pre", None):
+            out = call_layer(self.test_pre, x, targets=y, features=x, testing=True)
+            if isinstance(out, Prediction):
+                x, y = out.outputs, out.targets
+            else:
+                x = out
+
         outputs = self.call_train_test(x, y, sample_weight=sample_weight, testing=True)
 
         if getattr(self, "pre_eval_topk", None) is not None:
@@ -607,17 +684,43 @@ class BaseModel(tf.keras.Model):
             outputs = self.pre_eval_topk.call_outputs(outputs)
 
         self.compute_loss(x, outputs.targets, outputs.predictions, outputs.sample_weight)
-        metrics = self.compute_metrics(outputs, training=False)
+
+        metrics = self.compute_metrics(outputs)
+
         # Adding regularization loss to metrics
         metrics["regularization_loss"] = tf.reduce_sum(cast_losses_to_common_dtype(self.losses))
 
         return metrics
 
+    def predict_step(self, data):
+        x, _, _ = unpack_x_y_sample_weight(data)
+
+        if getattr(self, "predict_pre", None):
+            out = call_layer(self.predict_pr, x, features=x, training=False)
+            if isinstance(out, Prediction):
+                x = out.outputs
+            else:
+                x = out
+
+        return self(x, training=False)
+
     @tf.function
+    def train_compute_metrics(self, outputs: PredictionOutput, compiled_metrics: MetricsContainer):
+        """Returns metrics for the outputs of this step.
+
+        Re-computing metrics every `train_metrics_steps` steps.
+        """
+        # Compiled_metrics as an argument here because it is re-defined by `model.compile()`
+        # And checking `self.compiled_metrics` inside this function results in a reference to
+        # a deleted version of `compiled_metrics` if the model is re-compiled.
+        if self._should_compute_train_metrics_for_batch:
+            return self.compute_metrics(outputs, compiled_metrics)
+        return self.metrics_results()
+
     def compute_metrics(
         self,
         prediction_outputs: PredictionOutput,
-        training: bool,
+        compiled_metrics: Optional[MetricsContainer] = None,
     ) -> Dict[str, tf.Tensor]:
         """Overrides Model.compute_metrics() for some custom behaviour
            like compute metrics each N steps during training
@@ -627,38 +730,36 @@ class BaseModel(tf.keras.Model):
         ----------
         prediction_outputs : PredictionOutput
             Contains properties with targets and predictions
-        training : bool
-            Flag that indicates if metrics are being computed during
-            training or evaluation
+        compiled_metrics : MetricsContainer
+            The metrics container to compute metrics on.
+            If not provided, uses self.compiled_metrics
 
         Returns
         -------
         Dict[str, tf.Tensor]
             Dict with the metrics values
         """
+        if compiled_metrics is None:
+            compiled_metrics = self.compiled_metrics
 
-        should_compute_metrics = self._should_compute_train_metrics_for_batch or not training
-        if should_compute_metrics:
+        # This ensures that compiled metrics are built
+        # to make self.compiled_metrics.metrics available
+        if not compiled_metrics.built:
+            compiled_metrics.build(prediction_outputs.predictions, prediction_outputs.targets)
 
-            # This ensures that compiled metrics are built
-            # to make self.compiled_metrics.metrics available
-            if not self.compiled_metrics.built:
-                self.compiled_metrics.build(
-                    prediction_outputs.predictions, prediction_outputs.targets
-                )
+        # Providing label_relevant_counts for TopkMetrics, as metric.update_state()
+        # should have standard signature for better compatibility with Keras methods
+        # like self.compiled_metrics.update_state()
+        if hasattr(prediction_outputs, "label_relevant_counts"):
+            for topk_metric in filter_topk_metrics(compiled_metrics.metrics):
+                topk_metric.label_relevant_counts = prediction_outputs.label_relevant_counts
 
-            # Providing label_relevant_counts for TopkMetrics, as metric.update_state()
-            # should have standard signature for better compatibility with Keras methods
-            # like self.compiled_metrics.update_state()
-            if hasattr(prediction_outputs, "label_relevant_counts"):
-                for topk_metric in filter_topk_metrics(self.compiled_metrics.metrics):
-                    topk_metric.label_relevant_counts = prediction_outputs.label_relevant_counts
+        compiled_metrics.update_state(
+            prediction_outputs.targets,
+            prediction_outputs.predictions,
+            prediction_outputs.sample_weight,
+        )
 
-            self.compiled_metrics.update_state(
-                prediction_outputs.targets,
-                prediction_outputs.predictions,
-                prediction_outputs.sample_weight,
-            )
         # Returns the current value of metrics
         metrics = self.metrics_results()
         return metrics
@@ -703,12 +804,13 @@ class BaseModel(tf.keras.Model):
         workers=1,
         use_multiprocessing=False,
         train_metrics_steps=1,
+        pre=None,
         **kwargs,
     ):
         x = _maybe_convert_merlin_dataset(x, batch_size, **kwargs)
 
         # Bind schema from dataset to model in case we can't infer it from the inputs
-        if isinstance(x, BatchedDataset):
+        if isinstance(x, Loader):
             self.schema = x.schema
 
         validation_data = _maybe_convert_merlin_dataset(
@@ -719,10 +821,19 @@ class BaseModel(tf.keras.Model):
         fit_kwargs = {
             k: v
             for k, v in locals().items()
-            if k not in ["self", "kwargs", "train_metrics_steps", "__class__"]
+            if k not in ["self", "kwargs", "train_metrics_steps", "pre"] and not k.startswith("__")
         }
 
-        return super().fit(**fit_kwargs)
+        if pre:
+            self._reset_compile_cache()
+            self.train_pre = pre
+
+        out = super().fit(**fit_kwargs)
+
+        if pre:
+            del self.train_pre
+
+        return out
 
     def _add_metrics_callback(self, callbacks, train_metrics_steps):
         if callbacks is None:
@@ -753,11 +864,16 @@ class BaseModel(tf.keras.Model):
         workers=1,
         use_multiprocessing=False,
         return_dict=False,
+        pre=None,
         **kwargs,
     ):
         x = _maybe_convert_merlin_dataset(x, batch_size, shuffle=False, **kwargs)
 
-        return super().evaluate(
+        if pre:
+            self._reset_compile_cache()
+            self.test_pre = pre
+
+        out = super().evaluate(
             x,
             y,
             batch_size,
@@ -772,6 +888,11 @@ class BaseModel(tf.keras.Model):
             **kwargs,
         )
 
+        if pre:
+            del self.test_pre
+
+        return out
+
     def predict(
         self,
         x,
@@ -782,11 +903,16 @@ class BaseModel(tf.keras.Model):
         max_queue_size=10,
         workers=1,
         use_multiprocessing=False,
+        pre=None,
         **kwargs,
     ):
         x = _maybe_convert_merlin_dataset(x, batch_size, shuffle=False, **kwargs)
 
-        return super(BaseModel, self).predict(
+        if pre:
+            self._reset_compile_cache()
+            self.predict_pre = pre
+
+        out = super(BaseModel, self).predict(
             x,
             batch_size=batch_size,
             verbose=verbose,
@@ -797,6 +923,11 @@ class BaseModel(tf.keras.Model):
             use_multiprocessing=use_multiprocessing,
         )
 
+        if pre:
+            del self.predict_pre
+
+        return out
+
     def batch_predict(
         self, dataset: merlin.io.Dataset, batch_size: int, **kwargs
     ) -> merlin.io.Dataset:
@@ -862,7 +993,7 @@ class Model(BaseModel):
 
     def _maybe_build(self, inputs):
         if isinstance(inputs, dict):
-            _ragged_inputs = AsRaggedFeatures()(inputs)
+            _ragged_inputs = ListToRagged()(inputs)
             feature_shapes = {k: v.shape for k, v in _ragged_inputs.items()}
             feature_dtypes = {k: v.dtype for k, v in _ragged_inputs.items()}
 
@@ -909,7 +1040,7 @@ class Model(BaseModel):
 
     def call(self, inputs, targets=None, training=False, testing=False, output_context=False):
         context = self._create_context(
-            AsRaggedFeatures()(inputs),
+            ListToRagged()(inputs),
             targets=targets,
             training=training,
             testing=testing,
@@ -953,7 +1084,11 @@ class Model(BaseModel):
         if isinstance(outputs, Prediction):
             targets = outputs.targets if outputs.targets is not None else context.targets
             features = outputs.features if outputs.features is not None else context.features
-            outputs = outputs[0]
+            if isinstance(child, ModelOutput):
+                if not (context.training or context.testing):
+                    outputs = outputs[0]
+            else:
+                outputs = outputs[0]
             context = context.with_updates(targets=targets, features=features)
 
         return outputs, context
@@ -1397,13 +1532,156 @@ class RetrievalModel(Model):
         return recommender
 
 
+@tf.keras.utils.register_keras_serializable(package="merlin_models")
+class RetrievalModelV2(Model):
+    def __init__(
+        self,
+        *,
+        query: Union[Encoder, tf.keras.layers.Layer],
+        output: Union[ModelOutput, tf.keras.layers.Layer],
+        candidate: Optional[Union[Encoder, tf.keras.layers.Layer]] = None,
+        query_name="query",
+        candidate_name="candidate",
+        pre: Optional[tf.keras.layers.Layer] = None,
+        post: Optional[tf.keras.layers.Layer] = None,
+        **kwargs,
+    ):
+        if isinstance(output, ContrastiveOutput):
+            query_name = output.query_name
+            candidate_name = output.candidate_name
+
+        if query and candidate:
+            encoder = ParallelBlock({query_name: query, candidate_name: candidate})
+        else:
+            encoder = query
+
+        super().__init__(encoder, output, pre=pre, post=post, **kwargs)
+
+        self._query_name = query_name
+        self._candidate_name = candidate_name
+        self._encoder = encoder
+        self._output = output
+
+    def query_embeddings(
+        self,
+        dataset: Optional[merlin.io.Dataset] = None,
+        index: Optional[Union[str, ColumnSchema, Schema, Tags]] = None,
+        **kwargs,
+    ) -> merlin.io.Dataset:
+        query = self.query_encoder if self.has_candidate_encoder else self.encoder
+
+        if dataset is not None and hasattr(query, "encode"):
+            return query.encode(dataset, index=index, **kwargs)
+
+        if hasattr(query, "to_dataset"):
+            return query.to_dataset(**kwargs)
+
+        return query.encode(dataset, index=index, **kwargs)
+
+    def candidate_embeddings(
+        self,
+        dataset: Optional[merlin.io.Dataset] = None,
+        index: Optional[Union[str, ColumnSchema, Schema, Tags]] = None,
+        **kwargs,
+    ) -> merlin.io.Dataset:
+        if self.has_candidate_encoder:
+            candidate = self.candidate_encoder
+
+            if dataset is not None and hasattr(candidate, "encode"):
+                return candidate.encode(dataset, index=index, **kwargs)
+
+            if hasattr(candidate, "to_dataset"):
+                return candidate.to_dataset(**kwargs)
+
+            return candidate.encode(dataset, index=index, **kwargs)
+
+        if isinstance(self.last, ContrastiveOutput):
+            return self.last.to_dataset()
+
+        raise Exception(...)
+
+    @property
+    def encoder(self):
+        return self._encoder
+
+    @property
+    def has_candidate_encoder(self):
+        return (
+            isinstance(self.encoder, ParallelBlock)
+            and self._candidate_name in self.encoder.parallel_dict
+        )
+
+    @property
+    def query_encoder(self) -> Encoder:
+        if self.has_candidate_encoder:
+            output = self.encoder[self._query_name]
+        else:
+            output = self.encoder
+
+        output = self._check_encoder(output)
+
+        return output
+
+    @property
+    def candidate_encoder(self) -> Encoder:
+        output = None
+        if self.has_candidate_encoder:
+            output = self.encoder[self._candidate_name]
+
+        if output:
+            return self._check_encoder(output)
+
+        raise ValueError("No candidate encoder found.")
+
+    def _check_encoder(self, maybe_encoder):
+        output = maybe_encoder
+
+        from merlin.models.tf.core.encoder import Encoder
+
+        if isinstance(output, SequentialBlock):
+            output = Encoder(*maybe_encoder.layers)
+
+        if not isinstance(output, Encoder):
+            raise ValueError(f"Query encoder should be an Encoder, got {type(output)}")
+
+        return output
+
+    @classmethod
+    def from_config(cls, config, custom_objects=None):
+        pre = config.pop("pre", None)
+        if pre is not None:
+            pre = tf.keras.layers.deserialize(pre, custom_objects=custom_objects)
+
+        post = config.pop("post", None)
+        if post is not None:
+            post = tf.keras.layers.deserialize(post, custom_objects=custom_objects)
+
+        encoder = config.pop("_encoder", None)
+        if encoder is not None:
+            encoder = tf.keras.layers.deserialize(encoder, custom_objects=custom_objects)
+
+        output = config.pop("_output", None)
+        if output is not None:
+            output = tf.keras.layers.deserialize(output, custom_objects=custom_objects)
+
+        output = RetrievalModelV2(query=encoder, output=output, pre=pre, post=post)
+        output.__class__ = cls
+
+        return output
+
+    def get_config(self):
+        config = maybe_serialize_keras_objects(self, {}, ["pre", "post", "_encoder", "_output"])
+
+        return config
+
+
 def _maybe_convert_merlin_dataset(data, batch_size, shuffle=True, **kwargs):
     # Check if merlin-dataset is passed
     if hasattr(data, "to_ddf"):
         if not batch_size:
             raise ValueError("batch_size must be specified when using merlin-dataset.")
 
-        data = BatchedDataset(data, batch_size=batch_size, shuffle=shuffle, **kwargs)
+        data = Loader(data, batch_size=batch_size, shuffle=shuffle, **kwargs)
 
         if not shuffle:
             kwargs.pop("shuffle", None)
