@@ -20,6 +20,7 @@ import numpy as np
 import tensorflow as tf
 from keras.utils.tf_inspect import getfullargspec
 from packaging import version
+from tensorflow.python import to_dlpack
 
 from merlin.core.dispatch import DataFrameType
 from merlin.io import Dataset
@@ -58,15 +59,10 @@ def get_output_sizes_from_schema(schema, batch_size=0, max_sequence_length=None)
 
 
 def calculate_batch_size_from_input_shapes(input_shapes):
-    values = []
-
-    for val in input_shapes.values():
-        if isinstance(val, tuple) and isinstance(val[0], tf.TensorShape):
-            values.append(val[1])
-        else:
-            values.append(val)
-
-    return values[0][0]
+    val = list(input_shapes.values())[0]
+    if isinstance(val, tuple) and isinstance(val[1], tf.TensorShape):
+        val = val[1]
+    return val[0]
 
 
 def maybe_serialize_keras_objects(
@@ -279,6 +275,40 @@ def df_to_tensor(gdf, dtype=None):
     return x
 
 
+def tensor_to_df(tensor, index=None, gpu=None):
+    if gpu is None:
+        try:
+            import cudf  # noqa: F401
+            import cupy
+
+            gpu = True
+        except ImportError:
+            gpu = False
+
+    if gpu:
+        # Note: It is not possible to convert Tensorflow tensors to the cudf dataframe
+        # directly using dlPack (as the example commented below) because cudf.from_dlpack()
+        # expects the 2D tensor to be in Fortran order (column-major), which is not
+        # supported by TF (https://github.com/rapidsai/cudf/issues/10754).
+        # df = cudf.from_dlpack(to_dlpack(tf.convert_to_tensor(embeddings)))
+        tensor_cupy = cupy.fromDlpack(to_dlpack(tf.convert_to_tensor(tensor)))
+        df = cudf.DataFrame(tensor_cupy)
+        df.columns = [str(col) for col in list(df.columns)]
+        if not index:
+            index = cudf.RangeIndex(0, tensor.shape[0])
+        df.set_index(index)
+    else:
+        import pandas as pd
+
+        df = pd.DataFrame(tensor.numpy())
+        df.columns = [str(col) for col in list(df.columns)]
+        if not index:
+            index = pd.RangeIndex(0, tensor.shape[0])
+        df.set_index(index)
+
+    return df
+
+
 def add_epsilon_to_zeros(tensor: tf.Tensor, epsilon: float = 1e-24) -> tf.Tensor:
     """Replaces zeros by adding a small epsilon value to them.
     This is useful to avoid inf and nan errors on math ops
