@@ -32,15 +32,21 @@ def test_func(tb):
         "loss",
         "regularization_loss",
     ]
+    metrics_mlp = tb.ref("metrics_wide_n_deep")
+    assert sorted(list(metrics_mlp.keys())) == [
+        "auc_2",
+        "loss",
+        "regularization_loss",
+    ]
     metrics_dlrm = tb.ref("metrics_dlrm")
     assert sorted(list(metrics_dlrm.keys())) == [
-        "auc_2",
+        "auc_3",
         "loss",
         "regularization_loss",
     ]
     metrics_dcn = tb.ref("metrics_dcn")
     assert sorted(list(metrics_dcn.keys())) == [
-        "auc_3",
+        "auc_4",
         "loss",
         "regularization_loss",
     ]
