@@ -23,7 +23,7 @@ from tensorflow.keras.initializers import RandomUniform
 import merlin.models.tf as mm
 from merlin.io import Dataset
 from merlin.models.tf.utils import testing_utils
-from merlin.models.tf.utils.testing_utils import model_test
+from merlin.models.tf.utils.testing_utils import assert_output_shape, model_test
 from merlin.schema import ColumnSchema, Schema, Tags
 
 
@@ -87,7 +87,7 @@ class TestEmbeddingTable:
         layer = mm.EmbeddingTable(dim, column_schema, **kwargs)
 
         output = layer(inputs)
-        self._assert_output_shape(output, expected_output_shape)
+        assert_output_shape(output, expected_output_shape)
 
         if "sequence_combiner" in kwargs:
             assert isinstance(output, tf.Tensor)
@@ -102,23 +102,7 @@ class TestEmbeddingTable:
         assert copied_layer.input_dim == layer.input_dim
 
         output = copied_layer(inputs)
-        self._assert_output_shape(output, expected_output_shape)
-
-    def _get_shape(self, tensor_or_shape) -> tf.TensorShape:
-        if hasattr(tensor_or_shape, "shape"):
-            output_shape = tensor_or_shape.shape
-        else:
-            output_shape = tensor_or_shape
-        return output_shape
-
-    def _assert_output_shape(self, output, expected_output_shape):
-        if isinstance(expected_output_shape, dict):
-            for key in expected_output_shape.keys():
-                output_shape = self._get_shape(output[key])
-                assert list(output_shape) == list(expected_output_shape[key])
-        else:
-            output_shape = self._get_shape(output)
-            assert list(output_shape) == list(expected_output_shape)
+        assert_output_shape(output, expected_output_shape)
 
     def test_layer_simple(self):
         col_schema = self.sample_column_schema
@@ -146,7 +130,7 @@ class TestEmbeddingTable:
         column_schema = self.sample_column_schema
         layer = mm.EmbeddingTable(10, column_schema, **kwargs)
         output_shape = layer.compute_output_shape(input_shape)
-        self._assert_output_shape(output_shape, expected_output_shape)
+        assert_output_shape(output_shape, expected_output_shape)
 
     def test_dense_with_combiner(self):
         dim = 16
@@ -273,6 +257,65 @@ class TestEmbeddingTable:
             mm.EmbeddingTable(dim, col_schema_a, col_schema_b)
         assert "does not match existing input dim" in str(exc_info.value)
 
+    def test_select_by_tag(self):
+        dim = 4
+
+        col_schema_a = ColumnSchema(
+            "a",
+            dtype=np.int32,
+            properties={"domain": {"min": 0, "max": 10}},
+            tags=[Tags.USER, Tags.CATEGORICAL],
+        )
+        col_schema_b = ColumnSchema(
+            "b",
+            dtype=np.int32,
+            properties={"domain": {"min": 0, "max": 10}},
+            tags=[Tags.USER, Tags.CATEGORICAL],
+        )
+        col_schema_c = ColumnSchema(
+            "c",
+            dtype=np.int32,
+            properties={"domain": {"min": 0, "max": 10}},
+            tags=[Tags.ITEM, Tags.CATEGORICAL],
+        )
+
+        embedding_table = mm.EmbeddingTable(dim, col_schema_a, col_schema_b, col_schema_c)
+
+        categorical = embedding_table.select_by_tag(Tags.CATEGORICAL)
+        assert isinstance(categorical, mm.EmbeddingTable)
+        assert sorted(categorical.features) == ["a", "b", "c"]
+
+        inputs = {
+            "a": tf.constant([[0], [1], [2]]),
+            "b": tf.constant([[3], [4], [5]]),
+            "c": tf.constant([[6], [7], [8]]),
+        }
+
+        _ = categorical(inputs)
+        _ = embedding_table(inputs)
+
+        assert np.allclose(
+            categorical.table.embeddings.numpy(), embedding_table.table.embeddings.numpy()
+        )
+        assert categorical.table is embedding_table.table
+
+        assert embedding_table.select_by_tag(Tags.CONTINUOUS) is None
+
+        user = embedding_table.select_by_tag(Tags.USER)
+        assert isinstance(user, mm.EmbeddingTable)
+        assert sorted(user.features) == ["a", "b"]
+
+        _ = user(inputs)
+        assert np.allclose(
+            categorical.table.embeddings.numpy(), embedding_table.table.embeddings.numpy()
+        )
+        assert user.table is embedding_table.table
+
+        item = embedding_table.select_by_tag(Tags.ITEM)
+        assert isinstance(item, mm.EmbeddingTable)
+        assert sorted(item.features) == ["c"]
+        assert item.table is embedding_table.table
+
 
 @pytest.mark.parametrize("trainable", [True, False])
 def test_pretrained_from_InputBlockV2(trainable, music_streaming_data: Dataset):
@@ -293,7 +336,7 @@ def test_pretrained_from_InputBlockV2(trainable, music_streaming_data: Dataset):
         trainable={"item_id": trainable},
         dim=embed_dims,
     )
-    input_block = mm.InputBlockV2(music_streaming_data.schema, embeddings=embeddings_block)
+    input_block = mm.InputBlockV2(music_streaming_data.schema, categorical=embeddings_block)
 
     model = mm.DCNModel(
         music_streaming_data.schema,
